# Copyright (c) 2013 by pytest_pyramid authors and contributors
# <see AUTHORS file>
#
# This module is part of pytest_pyramid and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""pytest_pyramid main module."""

__version__ = '0.2.0'  # pragma: no cover



# Copyright (c) 2013 by pytest_pyramid authors and contributors
# <see AUTHORS file>
#
# This module is part of pytest_pyramid and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Plugin's definition and basic fixtures."""

from pytest_pyramid import factories


def pytest_addoption(parser):
    """Pytest option configurator."""
    parser.addoption(
        '--pc',
        action='store',
        default=None,
        metavar='path',
        dest='pyramid_config',
    )


# fixtures (made out of factories)
pyramid_config = factories.pyramid_config()
pyramid_app = factories.pyramid_app('pyramid_config')



# Copyright (c) 2013 by pytest_pyramid authors and contributors
# <see AUTHORS file>
#
# This module is part of pytest_pyramid and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Pytest's fixture factories."""

import os
import pytest
from webtest import TestApp
from pyramid.config import Configurator
try:
    from ConfigParser import ConfigParser
except ImportError:  # py3
    from configparser import ConfigParser


def pyramid_config(settings=None, config_path=None):
    """
    A pyramid_config fixture factory.

    Used to create aditional fixtures returning pyramid's
    :class:`~pyramid.config.Configurator` object.

    :param dict settings: optional parameter delivering settings for building
        a Configurator object.
    :param str config_path: optional parameter delivering path to pyramid
        configuration file

    :returns: configuration
    :rtype: `pyramid.config.Configurator`
    """
    @pytest.fixture
    def pyramid_config(request):
        app_settings = settings
        if app_settings is None:

            def load_settings(cpath, io_settings):
                config_parser = ConfigParser()
                config_parser.read(cpath)

                use = config_parser.get('app:main', 'use')
                if use and use.startswith('config:'):
                    base = os.path.dirname(cpath)
                    filename = use.replace('config:', '').strip()
                    sub_path = os.path.join(base, filename)
                    load_settings(os.path.abspath(sub_path), io_settings)

                for option, value in config_parser.items('app:main'):
                    io_settings[option] = value

            # load the application settings
            app_settings = {}
            cpath = config_path or request.config.getvalue('pyramid_config')
            load_settings(cpath, app_settings)

        return Configurator(settings=app_settings)

    return pyramid_config


def pyramid_app(config_fixture_name):
    """
    pyramid_app fixture factory.

    Creates a TestApp instance based on.

    :param str config_fixture_name: name of a fixture creating
        :class:`pyramid.config.Configurator`

    :returns: TestApp based on pyramid's COnfigurator object as returned from
        fixture passed by  **config_fixture_name**
    :rtype: webtest.app.TestApp
    """
    @pytest.fixture
    def pyramid_app(request):
        config = request.getfuncargvalue(config_fixture_name)

        app = TestApp(config.make_wsgi_app())

        return app

    return pyramid_app



"""Define dummy app for tests."""
from pyramid.config import Configurator


def main(global_config, **settings):
    """Dummy app configurator."""
    config = Configurator(settings)
    return config



"""Testing fixtures."""
from webtest import TestApp
from pyramid.config import Configurator
from pytest_pyramid import factories


def test_pyramid_app(pyramid_app):
    """
    Make sure we have everything needed for pyramid integration tests running.

    Check if:
    - app is instance of TestApp
    - we get proper status code from not defined route
    """
    assert isinstance(pyramid_app, TestApp)

    res = pyramid_app.get('/', status=404)
    assert res.status_code == 404


pyramid_config_path = factories.pyramid_config(
    config_path='tests/pyramid.test.ini'
)
pyramid_config_inheritance = factories.pyramid_config(
    config_path='tests/pyramid.use.test.ini'
)

pyramid_config_settings = factories.pyramid_config(settings={'env': 'pytest'})


def test_pyramid_config(pyramid_config, pyramid_config_path):
    """Test whether both fixtures are generated correctly."""
    assert isinstance(pyramid_config, Configurator)
    assert isinstance(pyramid_config_path, Configurator)
    assert pyramid_config_path != pyramid_config
    assert pyramid_config_path.registry.settings ==\
        pyramid_config.registry.settings


def test_pyramid_config_settings(pyramid_config_settings):
    """Test checking creating Configurator based on settings."""
    assert isinstance(pyramid_config_settings, Configurator)
    assert 'env' in pyramid_config_settings.registry.settings
    assert pyramid_config_settings.registry.settings['env'] == 'pytest'


def test_pyramid_inheritance_config(
        pyramid_config_path, pyramid_config_inheritance):
    """
    Test reading inheriting config through pytest_pyramid.

    Given:
        - that one config inherits the other,
    When:
        - having an additional config option defined,
        and it is different from the inherited config defined
    Then:
        The final config will contain sum of cconfig options defined
        in both config, with the inheriting config having the highest
        priority values.
    """
    assert isinstance(pyramid_config_inheritance, Configurator)
    assert isinstance(pyramid_config_path, Configurator)

    assert pyramid_config_inheritance.registry.settings['inheriting'] ==\
        'I do not really set anything'
    assert 'inheriting' not in pyramid_config_path.registry.settings

    assert pyramid_config_inheritance.registry.settings['use'] !=\
        pyramid_config_path.registry.settings['use']

    assert len(pyramid_config_inheritance.registry.settings) ==\
        len(pyramid_config_path.registry.settings) + 1

    assert pyramid_config_inheritance.registry.settings['one_value'] == '1'



"""pytest_pyramid's installation file."""

import os
import re
from setuptools import setup, find_packages

here = os.path.dirname(__file__)
with open(os.path.join(here, 'pytest_pyramid', '__init__.py')) as v_file:
    package_version = re.compile(
        r".*__version__ = '(.*?)'", re.S).match(v_file.read()).group(1)


def read(fname):
    """Read file into string."""
    return open(os.path.join(here, fname)).read()

requirements = [
    'pytest',
    'pyramid',
    'webtest'
]

tests_require = [
    'pytest-cov'
]

extras_require = {
    'docs': ['sphinx'],
    'tests': tests_require
}

setup(
    name='pytest_pyramid',
    version=package_version,
    description='pytest pyramid providing basic fixtures for testing ' +
    'pyramid applications with pytest test suite',
    long_description=(
        read('README.rst') + '\n\n' + read('CHANGES.rst')
    ),
    keywords='pyramid pytest testing',
    author='Grzegorz Sliwinski',
    author_email='fizyk@fizyk.net.pl',
    url='https://github.com/fizyk/pytest_pyramid',
    license="MIT License",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Testing',
    ],
    packages=find_packages(),
    install_requires=requirements,
    tests_require=tests_require,
    test_suite='tests',
    entry_points={
        'pytest11': [
            'pytest_pyramid = pytest_pyramid.plugin'
        ]},
    include_package_data=True,
    zip_safe=False,
    extras_require=extras_require,
)
